package com.faraphel.tasks_valider.database.dao

import androidx.room.Dao
import androidx.room.Query
import com.faraphel.tasks_valider.database.dao.base.BaseDao
import com.faraphel.tasks_valider.database.entities.*

@Dao
interface PersonDao : BaseDao<PersonEntity> {
    @Query("SELECT * FROM ${PersonEntity.TABLE_NAME}")
    override fun getAll(): List<PersonEntity>

    /**
     * Get the object from its identifier
     */
    @Query("SELECT * FROM ${PersonEntity.TABLE_NAME} WHERE id = :id")
    fun getById(id: Long): PersonEntity

    /**
     * Allow to get all the classes the person is attending as a student
     */
    @Query(
        "SELECT * FROM ${ClassEntity.TABLE_NAME} " +
        "JOIN ${RelationClassPersonEntity.TABLE_NAME} " +
        "ON ${ClassEntity.TABLE_NAME}.id = ${RelationClassPersonEntity.TABLE_NAME}.student_id " +
        "WHERE ${RelationClassPersonEntity.TABLE_NAME}.student_id = :id"
    )
    fun getClasses(id: Long): List<ClassEntity>


    /**
     * Get all the tasks this user approved as a teacher
     * @param id the id of the person
     */
    @Query("SELECT * FROM ${ValidationEntity.TABLE_NAME} WHERE teacher_id = :id")
    fun getTasksApproved(id: Long): List<ValidationEntity>

    /**
     * Get all the tasks this user validated as a student
     * @param id the id of the person
     */
    @Query("SELECT * FROM ${ValidationEntity.TABLE_NAME} WHERE student_id = :id")
    fun getTasksValidated(id: Long): List<ValidationEntity>
}
