package com.faraphel.tasks_valider.ui.screen.communication.internet.server

import android.app.Activity
import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Button
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.text.input.KeyboardType
import androidx.room.Room
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.TaskServer
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.ui.screen.communication.DEFAULT_SERVER_PORT
import com.faraphel.tasks_valider.ui.screen.communication.RANGE_SERVER_PORT
import com.faraphel.tasks_valider.ui.screen.task.TaskSessionScreen


@Composable
fun CommunicationInternetServerScreen(activity: Activity) {
    val client = remember { mutableStateOf<TaskClient?>(null) }

    // if the server is not created, prompt the user for the server configuration
    if (client.value == null) CommunicationInternetServerContent(activity, client)
    // else, go to the base tasks screen
    else TaskSessionScreen(activity, client.value!!)
}


@Composable
fun CommunicationInternetServerContent(activity: Activity, client: MutableState<TaskClient?>) {
    val expandedStudentList = remember { mutableStateOf(false) }
    val serverPort = remember { mutableIntStateOf(DEFAULT_SERVER_PORT) }

    Column {
        // student list
        Button(onClick = { expandedStudentList.value = !expandedStudentList.value }) {
            Text(text = "Select Students List")
        }
        DropdownMenu(
            expanded = expandedStudentList.value,
            onDismissRequest = { expandedStudentList.value = false }
        ) {
            DropdownMenuItem(
                text = { Text("ISRI") },
                onClick = {}
            )
            DropdownMenuItem(
                text = { Text("MIAGE") },
                onClick = {}
            )
            // TODO(Faraphel): student lists should be loaded from the database or a file
        }

        // server port
        TextField(
            value = serverPort.intValue.toString(),
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Number),
            onValueChange = { text ->
                val port = text.toInt()
                if (port in RANGE_SERVER_PORT) {
                    serverPort.intValue = port
                }
            }
        )

        Button(onClick = {
            // Reset the database | TODO(Faraphel): only for testing purpose
            activity.deleteDatabase("local")

            // Create the database
            val database = Room.databaseBuilder(
                activity,
                TaskDatabase::class.java,
                "local"
            ).build()

            // Create the server
            Log.i("room-server", "creating the server")
            Thread {  // a thread is used for networking
                val server = TaskServer(serverPort.intValue, database)
                server.start()

                // Get the client from the server
                client.value = server.getClientAdmin()
            }.start()
        }) {
            Text("Create")
        }
    }
}
